package example.graphicalEditor;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;

/**
 * A standard Swing GUI for a graphical player. This editor
 * is not fully functional, its interface has just been 
 * designed for illustration purpose.
 * 
 * @author Caroline Appert
 * @see example.graphicalEditor#StrokeGraphicalEditor
 */
public class SimpleGraphicalEditor extends JFrame {

	protected GraphicalEditor editor;
	protected JDialog dialogAbout;

	public SimpleGraphicalEditor() {
		super();
		editor = new GraphicalEditor(2000, 2000);
		JScrollPane jsp = new JScrollPane(editor);
		jsp.setPreferredSize(new Dimension(500, 500));

		buildDialogAbout();
		
		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		JMenu general = new JMenu("Graphical Editor");
		JMenuItem clear = new JMenuItem("Clear");
		clear.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				editor.clear();
			}
		});
		JMenuItem about = new JMenuItem("About");
		about.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dialogAbout.setVisible(true);
			}
		});
		JMenuItem quit = new JMenuItem("Quit");
		quit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
		});
		general.add(clear);
		general.add(about);
		general.add(quit);
		menuBar.add(general);
		JMenu edit = new JMenu("Edit");
		JMenuItem cut = new JMenuItem("Cut");
		cut.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				editor.cut();
			}
		});
		cut.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_X, java.awt.Event.META_MASK));
		JMenuItem copy = new JMenuItem("Copy");
		copy.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				editor.copy();
			}
		});
		copy.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C, java.awt.Event.META_MASK));
		JMenuItem paste = new JMenuItem("Paste");
		paste.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				editor.paste();
			}
		});
		paste.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_V, java.awt.Event.META_MASK));
		edit.add(cut);
		edit.add(copy);
		edit.add(paste);
		menuBar.add(edit);

		JToolBar toolbar = new JToolBar();
		toolbar.add(new AbstractAction("Rectangle tool", new ImageIcon("icons"+File.separator+"tool_rect.png")) {
			public void actionPerformed(ActionEvent e) {
				editor.setTool(GraphicalEditor.TOOL_RECT);
			}
		});
		toolbar.add(new AbstractAction("Ellipse tool", new ImageIcon("icons"+File.separator+"tool_ell.png")) {
			public void actionPerformed(ActionEvent e) {
				editor.setTool(GraphicalEditor.TOOL_ELL);
			}
		});

		add(toolbar, BorderLayout.PAGE_START);
		add(jsp, BorderLayout.CENTER);

		pack();
		setVisible(true);
	}

	protected void buildDialogAbout() {
		dialogAbout = new JDialog(this, "About");
		dialogAbout.setLocationRelativeTo(this);

		JLabel jLabelAbout = new JLabel("A Simple Music Player in Java Swing", JLabel.CENTER);
		JButton jButtonOkAbout = new JButton("Ok");
		jButtonOkAbout.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dialogAbout.setVisible(false);
			}
		});

		GridBagLayout dialogLayout = new GridBagLayout();
		dialogAbout.getContentPane().setLayout(dialogLayout);
		GridBagConstraints constraints = new GridBagConstraints();
		buildConstraints(constraints, 0, 0, 3, 1, 100, 60, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
		constraints.insets = new Insets(5, 5, 5, 5);
		dialogAbout.getContentPane().add(jLabelAbout, constraints);
		buildConstraints(constraints, 2, 1, 1, 1, 30, 40, GridBagConstraints.NONE, GridBagConstraints.SOUTHEAST);
		constraints.insets = new Insets(5, 5, 5, 5);
		dialogAbout.getContentPane().add(jButtonOkAbout, constraints);
		dialogAbout.pack();
	}

	static void buildConstraints(GridBagConstraints constraints, int x, int y, int w, int h, double columnExpand, double rowExpand, int componentFill, int componentAnchor) {
		constraints.gridx = x;
		constraints.gridy = y;
		constraints.gridwidth = w;
		constraints.gridheight = h;
		constraints.weightx = columnExpand;
		constraints.weighty = rowExpand;
		constraints.fill = componentFill;
		constraints.anchor = componentAnchor;
	}
	
	public static void main(String[] args) {
		new SimpleGraphicalEditor();
	}
}
