package example.graphicalEditor;

import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;

import strokes.StrokeShortcuts;
import strokes.criterions.ClickCriterion;
import strokes.criterions.Criterion;
import strokes.criterions.DefaultCriterion;
import strokes.criterions.FlickCriterion;
import strokes.designShortcuts.DesignShortcuts;
import strokes.listeners.StrokeAdapter;
import strokes.listeners.StrokeEvent;
import strokes.transitions.ShortcutRecognized;
import strokes.transitions.StrokeNotRecognized;
import strokes.transitions.StrokeRecognized;
import fr.lri.swingstates.canvas.CIntentionalTag;
import fr.lri.swingstates.canvas.CShape;
import fr.lri.swingstates.canvas.CTag;
import fr.lri.swingstates.gestures.Gesture;
import fr.lri.swingstates.gestures.GestureUtils;
import fr.lri.swingstates.sm.State;
import fr.lri.swingstates.sm.StateMachine;
import fr.lri.swingstates.sm.Transition;

/**
 * A standard Swing GUI for a graphical editor
 * augmented with stroke shortcuts. This example
 * is used in the <a href="http://www.lri.fr/~appert/StrokeShortcuts"><i>StrokeShortcuts</i>' tutorial</a>..
 * 
 * @author Caroline Appert
 */
public class StrokeGraphicalEditor extends SimpleGraphicalEditor {
	
	void select(final Rectangle2D strokeBoundingBox) {
		// get the shapes in the editor that intersect the stroke's bounding box
		CTag tag = new CIntentionalTag(editor) {
			public boolean criterion(CShape s) {
				if(!s.isPickable()) return false;
				boolean intersected = !(s.getMaxX() < strokeBoundingBox.getMinX() || s.getMinX() > strokeBoundingBox.getMaxX()
						|| s.getMaxY() < strokeBoundingBox.getMinY() || s.getMinY() > strokeBoundingBox.getMaxY());
				return intersected;
			};
		};
		tag.reset();
		
		// get the selected shapes in the editor
		CTag tg = editor.getTag(SelectionTag.class);
		while(tag.hasNext()) {
			CShape next = tag.nextShape();
			// select only the shapes that are not selected and intercept the stroke's bounding box
			if(!tg.tagsShape(next))
				next.addTag(new SelectionTag());
		}
	}
	
	public StrokeGraphicalEditor() {
		super();
		
		/****************************************************************/
		/************** Enable stroke shortcuts *************************/
		/****************************************************************/
		
		/* ***************************************************************
		 * Designing the mappings (command, stroke) using the 
		 * Design Shortcuts application. The set of shortcuts for this 
		 * music player application is saved in a file "player.strokes".
		 * ***************************************************************/
		
		/*
		DesignShortcuts designShortcuts = new DesignShortcuts();
		designShortcuts.addWindow(this);
		designShortcuts.setVisible(true);
		*/
		
		StrokeShortcuts manager = new StrokeShortcuts();
		manager.addWindow(this);
		
		manager.addShortcuts("graphicaleditor.strokes", true, true);
//		manager.addStrokeListener(new StrokeAdapter() {
//			public void shortcutRecognized(StrokeEvent event) {
//				if(event.getNameEvent().compareTo("Paste") == 0) {
//					editor.setPastePosition(event.getPointInComponent(editor));
//				}
//			}
//			public void strokeRecognized(StrokeEvent event) {
//				if(event.getNameEvent().compareTo("Lasso") == 0) {
//					select(GestureUtils.boundingBox(event.getStrokeInComponent(editor).getPoints()));
//				}
//			}
//		});
		manager.addStrokeListener(new StateMachine() {
			State start = new State() {
				Transition cut = new ShortcutRecognized("Paste", ">> start") {
					public void action() {
						editor.setPastePosition(getStrokeEvent().getPointInComponent(editor));
					}
				};
				Transition select = new StrokeRecognized("Lasso", ">> start") {
					public void action() {
						select(GestureUtils.boundingBox(getStrokeEvent().getStrokeInComponent(editor).getPoints()));
					}
				};
			};
		});
		
		Criterion criterion;
		
		// right mouse button
		criterion = new DefaultCriterion() {
			public boolean startStroke(MouseEvent event) {
				return event.getButton() == MouseEvent.BUTTON3;
			}
		};
		
		// flick criterion
//		manager.setMinimumStrokeLength(30);
//		criterion = new FlickCriterion(editor, manager);
		
//		// click criterion
//		criterion = new ClickCriterion(2, editor);
		
		manager.enableStrokes(this, true, true, criterion);
	}
	
	public static void main(String[] args) {
		new StrokeGraphicalEditor();
	}
	
}