package example.player;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;

/**
 * A standard Swing GUI for a music player. This player
 * is not fully functional, its interface has just been 
 * designed for illustration purpose.
 * 
 * @author Caroline Appert
 * @see example.player#StrokePlayer
 */
public class SimplePlayer extends javax.swing.JFrame {

	protected JFileChooser jFileChooserPlayList;
	protected JFileChooser jFileChooserImport;
	
	protected JDialog dialogAbout;
	protected JSlider sliderSong;
	protected JSlider sliderVolume;
	protected JList playlist;
	protected DefaultComboBoxModel playlistModel;
	
	protected AbstractAction actionImport;
	protected AbstractAction actionSave;
	protected AbstractAction actionOpen;
	protected AbstractAction actionRewind;
	protected AbstractAction actionFastForward;
	protected AbstractAction actionPreviousSong;
	protected AbstractAction actionNextSong;
	protected AbstractAction actionVolumeDown;
	protected AbstractAction actionVolumeUp;
	protected AbstractAction actionPause;
	protected AbstractAction actionStop;
	protected AbstractAction actionPlay;
	protected AbstractAction actionRemoveSelectedSongs;

	protected Song currentSong;

	/**
	 * Auto-generated main method to display this JFrame
	 */
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				SimplePlayer inst = new SimplePlayer();
				inst.setLocationRelativeTo(null);
				inst.setVisible(true);
				inst.playlist.requestFocus();
			}
		});
	}

	public SimplePlayer() {
		super("Music player");
		initGUI();
	}
	
	protected void buildDialogAbout() {
		dialogAbout = new JDialog(this, "About");
		dialogAbout.setLocationRelativeTo(this);
		
		JLabel jLabelAbout = new JLabel("A Simple Music Player in Java Swing", JLabel.CENTER);
		JButton jButtonOkAbout = new JButton("Ok");
		jButtonOkAbout.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dialogAbout.setVisible(false);
			}
		});

		GridBagLayout dialogLayout = new GridBagLayout();
		dialogAbout.getContentPane().setLayout(dialogLayout);
		GridBagConstraints constraints = new GridBagConstraints();
		buildConstraints(constraints, 0, 0, 3, 1, 100, 60, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
		constraints.insets = new Insets(5, 5, 5, 5);
		dialogAbout.getContentPane().add(jLabelAbout, constraints);
		buildConstraints(constraints, 2, 1, 1, 1, 30, 40, GridBagConstraints.NONE, GridBagConstraints.SOUTHEAST);
		constraints.insets = new Insets(5, 5, 5, 5);
		dialogAbout.getContentPane().add(jButtonOkAbout, constraints);
		dialogAbout.pack();
	}
	
	protected JScrollPane buildPlayList() {
		playlistModel = new DefaultComboBoxModel();
		playlist = new JList(playlistModel);
		playlist.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if (e.getValueIsAdjusting() == false) {
					if (playlist.getSelectedIndex() == -1) {
						currentSong = null;
						// getActionPlay().setEnabled(false);
					} else {
						boolean started = false;
						if(currentSong != null) {
							started = currentSong.isStarted();
							currentSong.stop();
						}
						currentSong = new Song(sliderSong);
						//	getActionPlay().setEnabled(true);
						if(started) currentSong.play();
					}
				}
			}
		});
		playlist.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				if(e.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
					getActionRemoveSelectedSongs().actionPerformed(new ActionEvent(playlist, ActionEvent.ACTION_PERFORMED,"Remove selected songs"));
				}
			}
		});
		JScrollPane jsp = new JScrollPane(playlist);
		jsp.getHorizontalScrollBar().setEnabled(true);
		return jsp;
	}

	protected void initGUI() {
		try {
			GridBagLayout thisLayout = new GridBagLayout();
			GridBagConstraints constraints = new GridBagConstraints();
			
			setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			getContentPane().setLayout(thisLayout);
			buildConstraints(constraints, 0, 0, 1, 1, 100, 80, GridBagConstraints.BOTH, GridBagConstraints.NORTH);
			getContentPane().add(buildPlayList(), constraints);
			
			buildDialogAbout();
			
			JMenuBar menubar = new JMenuBar();
			setJMenuBar(menubar);
			JMenu jMenuPlayer = new JMenu("Player");
			menubar.add(jMenuPlayer);
			JMenuItem jMenuItemAbout = new JMenuItem("About");
			jMenuPlayer.add(jMenuItemAbout);
			jMenuItemAbout.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent evt) {
					dialogAbout.setVisible(true);
				}
			});
			JMenuItem jMenuItemHelp = new JMenuItem("Help");
			jMenuPlayer.add(jMenuItemHelp);
			jMenuItemHelp.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent evt) {
					System.out.println("jMenuItemPreferences.actionPerformed, event="+evt);
				}
			});
			JMenuItem jMenuItemPreferences = new JMenuItem("Preferences");
			jMenuPlayer.add(jMenuItemPreferences);
			jMenuItemPreferences.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent evt) {
					System.out.println("jMenuItemPreferences.actionPerformed, event="+evt);
				}
			});
			JMenuItem jMenuItemQuit = new JMenuItem("Quit");
			jMenuPlayer.add(jMenuItemQuit);
			jMenuItemQuit.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent evt) {
					System.exit(0);
				}
			});

			JMenu jMenuPlayList = new JMenu("PlayList");
			menubar.add(jMenuPlayList);
			JMenuItem jMenuItemOpen = new JMenuItem(getActionOpen());
			jMenuPlayList.add(jMenuItemOpen);
			JMenuItem jMenuItemSave = new JMenuItem(getActionSave());
			jMenuPlayList.add(jMenuItemSave);
			JMenuItem jMenuItemImport = new JMenuItem(getActionImport());
			jMenuPlayList.add(jMenuItemImport);
			JMenuItem jMenuItemRemove = new JMenuItem(getActionRemoveSelectedSongs());
			jMenuPlayList.add(jMenuItemRemove);

			JMenu jMenuSong = new JMenu("Song");
			menubar.add(jMenuSong);
			JMenuItem jMenuItemPlay = new JMenuItem(getActionPlay());
			jMenuSong.add(jMenuItemPlay);
			jMenuItemPlay.setIcon(null);
			jMenuItemPlay.setToolTipText("Plays the current selected song");
			JMenuItem jMenuItemPause = new JMenuItem(getActionPause());
			jMenuSong.add(jMenuItemPause);
			JMenuItem jMenuItemStop = new JMenuItem(getActionStop());
			jMenuSong.add(jMenuItemStop);
			JMenuItem jMenuItemFastForward = new JMenuItem(getActionFastForward());
			jMenuSong.add(jMenuItemFastForward);
			JMenuItem jMenuItemRewind = new JMenuItem(getActionRewind());
			jMenuSong.add(jMenuItemRewind);

			JMenu jMenuVolume = new JMenu("Controls");
			menubar.add(jMenuVolume);
			JMenuItem jMenuItemVolumeUp = new JMenuItem(getActionVolumeUp());
			jMenuVolume.add(jMenuItemVolumeUp);
			JMenuItem jMenuItemVolumeDown = new JMenuItem(getActionVolumeDown());
			jMenuVolume.add(jMenuItemVolumeDown);
			JMenuItem jMenuItemNextSong = new JMenuItem(getActionNextSong());
			jMenuVolume.add(jMenuItemNextSong);
			jMenuItemNextSong.setIcon(null);
			JMenuItem jMenuItemPreviousSong = new JMenuItem(getActionPreviousSong());
			jMenuVolume.add(jMenuItemPreviousSong);
			jMenuItemPreviousSong.setIcon(null);
			
			JPanel controlPanel = new JPanel();
			buildConstraints(constraints, 0, 1, 1, 1, 100, 20, GridBagConstraints.BOTH, GridBagConstraints.SOUTH);
			getContentPane().add(controlPanel, constraints);

			GridBagLayout controlPanelLayout = new GridBagLayout();
			controlPanel.setLayout(controlPanelLayout);
			controlPanel.setBackground(new java.awt.Color(236,236,236));
			sliderSong = new JSlider();
			BorderLayout jSliderSongLayout = new BorderLayout();
			sliderSong.setLayout(jSliderSongLayout);
			sliderSong.setValue(0);
			sliderSong.addMouseListener(new MouseAdapter() {
				public void mousePressed(MouseEvent e) {
					if(currentSong != null) currentSong.suspend();
				}
			});

			JButton jButtonPrevious = new JButton(getActionPreviousSong());
			jButtonPrevious.setText(null);
			jButtonPrevious.setBackground(new java.awt.Color(235,235,235));
			JButton jButtonPlay = new JButton(getActionPlay());
			jButtonPlay.setText(null);
			jButtonPlay.setToolTipText("Plays the current selected song");
			jButtonPlay.setBackground(new java.awt.Color(235,235,235));
			JButton jButtonNext = new JButton(getActionNextSong());
			jButtonNext.setText(null);
			jButtonNext.setBackground(new java.awt.Color(235,235,235));
			
			sliderVolume = new JSlider(JSlider.HORIZONTAL, 0, 30, 15);
			sliderVolume.setMajorTickSpacing(10);
			sliderVolume.setOpaque(true);
			sliderVolume.setBackground(new java.awt.Color(235,235,235));
			JLabel plus = new JLabel("+", JLabel.LEFT);
			plus.setMinimumSize(new Dimension(plus.getMinimumSize().width, jButtonPrevious.getPreferredSize().height));
			JLabel minus = new JLabel("-");
			minus.setIcon(new ImageIcon("icons"+File.separator+"volume_icon.png"));
			minus.setBackground(new java.awt.Color(235,235,235));
			minus.setMinimumSize(new Dimension(minus.getMinimumSize().width, sliderVolume.getPreferredSize().height));
			buildConstraints(constraints, 0, 0, 6, 1, 100, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTH);
			controlPanel.add(sliderSong, constraints);
			buildConstraints(constraints, 0, 1, 1, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.SOUTH);
			controlPanel.add(minus, constraints);
			buildConstraints(constraints, 1, 1, 1, 1, 80, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.SOUTH);
			controlPanel.add(sliderVolume, constraints);
			buildConstraints(constraints, 2, 1, 1, 1, 15, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.SOUTH);
			controlPanel.add(plus, constraints);
			buildConstraints(constraints, 3, 1, 1, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.SOUTH);
			controlPanel.add(jButtonPrevious, constraints);
			buildConstraints(constraints, 4, 1, 1, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.SOUTH);
			controlPanel.add(jButtonPlay, constraints);
			buildConstraints(constraints, 5, 1, 1, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.SOUTH);
			controlPanel.add(jButtonNext, constraints);
				
				
			
			
			pack();
			this.setSize(451, 446);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	protected AbstractAction getActionPlay() {
		if(actionPlay == null) {
			actionPlay = new AbstractAction("Play", new ImageIcon("icons"+File.separator+"play_icon.png")) {
				public void actionPerformed(ActionEvent evt) {
					if(currentSong != null) currentSong.play();
				}

			};
		}
		return actionPlay;
	}

	protected AbstractAction getActionStop() {
		if(actionStop == null) {
			actionStop = new AbstractAction("Stop", null) {
				public void actionPerformed(ActionEvent evt) {
					if(currentSong != null) currentSong.stop();
				}
			};
		}
		return actionStop;
	}

	protected AbstractAction getActionPause() {
		if(actionPause == null) {
			actionPause = new AbstractAction("Pause", null) {
				public void actionPerformed(ActionEvent evt) {
					if(currentSong != null) currentSong.pause();
				}
			};
		}
		return actionPause;
	}

	protected AbstractAction getActionVolumeUp() {
		if(actionVolumeUp == null) {
			actionVolumeUp = new AbstractAction("Volume Up", null) {
				public void actionPerformed(ActionEvent evt) {
					sliderVolume.setValue(sliderVolume.getValue() + 1);
				}
			};
		}
		return actionVolumeUp;
	}

	protected AbstractAction getActionVolumeDown() {
		if(actionVolumeDown == null) {
			actionVolumeDown = new AbstractAction("Volume Down", null) {
				public void actionPerformed(ActionEvent evt) {
					sliderVolume.setValue(sliderVolume.getValue() - 1);
				}
			};
		}
		return actionVolumeDown;
	}

	protected AbstractAction getActionNextSong() {
		if(actionNextSong == null) {
			actionNextSong = new AbstractAction("Next", new ImageIcon("icons"+File.separator+"next_icon.png")) {
				public void actionPerformed(ActionEvent evt) {
					int index = playlist.getSelectedIndex();
					if((index + 1) < playlistModel.getSize()) {
						boolean play = (currentSong != null && currentSong.isPlaying());
						if(currentSong != null) currentSong.stop();
						playlist.setSelectedIndex(index+1);
						currentSong = new Song(sliderSong);
						if(play) currentSong.play();
					}
					
				}
			};
		}
		return actionNextSong;
	}

	protected AbstractAction getActionPreviousSong() {
		if(actionPreviousSong == null) {
			actionPreviousSong = new AbstractAction("Previous", new ImageIcon("icons"+File.separator+"previous_icon.png")) {
				public void actionPerformed(ActionEvent evt) {
					int index = playlist.getSelectedIndex();
					if((index - 1) >= 0) {
						boolean play = (currentSong != null && currentSong.isPlaying());
						if(currentSong != null) currentSong.stop();
						playlist.setSelectedIndex(index-1);
						currentSong = new Song(sliderSong);
						if(play) currentSong.play();
					}
				}
			};
		}
		return actionPreviousSong;
	}

	protected AbstractAction getActionFastForward() {
		if(actionFastForward == null) {
			actionFastForward = new AbstractAction("Fast Forward", null) {
				public void actionPerformed(ActionEvent evt) {
					if(currentSong != null) {
						currentSong.fastForward();
					}
				}
			};
		}
		return actionFastForward;
	}

	protected AbstractAction getActionRewind() {
		if(actionRewind == null) {
			actionRewind = new AbstractAction("Rewind", null) {
				public void actionPerformed(ActionEvent evt) {
					if(currentSong != null) {
						currentSong.rewind();
					}
				}
			};
		}
		return actionRewind;
	}

	private FileFilter filterPL = new FileFilter() {
		public boolean accept(File f) {
			return (f.isDirectory() || f.getName().endsWith(".pl"));
		}
		public String getDescription() {
			return "playlist files";
		}

	};

	protected AbstractAction getActionOpen() {
		if(actionOpen == null) {
			actionOpen = new AbstractAction("Open playlist", null) {
				public void actionPerformed(ActionEvent evt) {
					if(jFileChooserPlayList == null) jFileChooserPlayList = new JFileChooser();
					jFileChooserPlayList.setFileFilter(filterPL);
					try {
						jFileChooserPlayList.setCurrentDirectory(new File(new File(".").getCanonicalPath()));
					} catch (IOException e1) {
						e1.printStackTrace();
					}
					int returnVal = jFileChooserPlayList.showOpenDialog(SimplePlayer.this);
					if(returnVal == JFileChooser.APPROVE_OPTION) {
						setPlayList(jFileChooserPlayList.getSelectedFile());
					}
				}

				private void setPlayList(File selectedFile) {
					try {
						BufferedReader br = new BufferedReader(new FileReader(selectedFile));
						String line = br.readLine();
						playlistModel.removeAllElements();
						while(line != null) {
							playlistModel.addElement(line);
							line = br.readLine();
						}
					} catch (FileNotFoundException e) {
						e.printStackTrace();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			};
		}
		return actionOpen;
	}

	protected AbstractAction getActionSave() {
		if(actionSave == null) {
			actionSave = new AbstractAction("Save playlist", null) {
				public void actionPerformed(ActionEvent evt) {
					if(jFileChooserPlayList == null) jFileChooserPlayList = new JFileChooser();
					jFileChooserPlayList.setFileFilter(filterPL);
					int returnVal = jFileChooserPlayList.showSaveDialog(SimplePlayer.this);
					if(returnVal == JFileChooser.APPROVE_OPTION) {
						savePlayList(jFileChooserPlayList.getSelectedFile());
					}
				}

				private void savePlayList(File selectedFile) {
					try {
						FileWriter fw = new FileWriter(selectedFile);
						for(int i = 0; i < playlistModel.getSize(); i++)
							fw.write(playlistModel.getElementAt(i)+"\n");
						fw.close();
					} catch (FileNotFoundException e) {
						e.printStackTrace();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			};
		}
		return actionSave;
	}

	protected AbstractAction getActionImport() {
		if(actionImport == null) {
			actionImport = new AbstractAction("Import Folder", null) {
				public void actionPerformed(ActionEvent evt) {
					if(jFileChooserImport == null) jFileChooserImport = new JFileChooser();
					jFileChooserImport.setFileFilter(filterMP3);
					try {
						jFileChooserImport.setCurrentDirectory(new File(new File(".").getCanonicalPath()));
					} catch (IOException e1) {
						e1.printStackTrace();
					}
					jFileChooserImport.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
					jFileChooserImport.setCurrentDirectory(new File("/Users/appert/Music/iTunes"));
					int returnVal = jFileChooserImport.showOpenDialog(SimplePlayer.this);
					if(returnVal == JFileChooser.APPROVE_OPTION) {
						addToPlayList(jFileChooserImport.getSelectedFile());
					}
				}

				FileFilter filterMP3 = new FileFilter() {
					public boolean accept(File f) {
						return (f.isDirectory() || f.getName().endsWith(".mp3"));
					}
					public String getDescription() {
						return "mp3 music files";
					}

				};

				FilenameFilter nameFilter = new FilenameFilter() {
					public boolean accept(File dir, String name) {
						return (dir.isDirectory() || name.endsWith(".mp3"));
					}
				};

				void addToPlayList(File f) {
					if(f.isDirectory()) {
						File[] files = f.listFiles(nameFilter);
						for (int j = 0; j < files.length; j++) {
							addToPlayList(files[j]);
						}
					} else {
						playlistModel.addElement(f.getName());
					}

				}
			};
		}
		return actionImport;
	}
	
	protected AbstractAction getActionRemoveSelectedSongs() {
		if(actionRemoveSelectedSongs == null) {
			actionRemoveSelectedSongs = new AbstractAction("Remove selected songs", null) {
				public void actionPerformed(ActionEvent evt) {
					int[] indices = playlist.getSelectedIndices();
					for(int i = (indices.length - 1); i >= 0 ; i--) {
						playlistModel.removeElementAt(indices[i]);
					}
				}
			};
		}
		return actionRemoveSelectedSongs;
	}

	static void buildConstraints(GridBagConstraints constraints, int x, int y, int w, int h, double columnExpand, double rowExpand, int componentFill, int componentAnchor) {
		constraints.gridx = x;
		constraints.gridy = y;
		constraints.gridwidth = w;
		constraints.gridheight = h;
		constraints.weightx = columnExpand;
		constraints.weighty = rowExpand;
		constraints.fill = componentFill;
		constraints.anchor = componentAnchor;
	}
}