package example.player;

import javax.swing.JSlider;

import fr.lri.swingstates.animations.Animation;

/**
 * A simple animation that simulates a song playing
 * by continuously changing the value of a JSlider.
 * 
 * @author Caroline Appert
 *
 */
class Song extends Animation {

	private JSlider slider;
	private double value;
	private int sign = 1;
	private boolean toStart = true;
	private boolean playing = false;
	private double pace = 0.1;
	
	public Song(JSlider slider) {
		super();
		this.slider = slider;
		setLapDuration(60000);
		slider.setValue(0);
		value = 0;
		sign = 1;
	}

	public void doStart() {
		slider.setValue(0);
		value = 0;
		sign = 1;
		toStart = false;
		playing = true;
	}

	public void doStop() {
		playing = false;
		toStart = true;
	}

	public void doResume() {
		if(toStart) start();
	}
	
	public void fastForward() {
		pace *= 2;
	}
	
	public void rewind() {
		sign = -1;
	}
	
	public void play() {
		sign = 1;
		pace = 0.1;
		playing = true;
		resume();
	}
	
	public void pause() {
		playing = false;
		suspend();
	}

	public void step(double t) {
		value += sign * pace ;
		if(value >= slider.getMaximum() || value <= slider.getMinimum()) stop();
		else slider.setValue((int)value);
	}

	public boolean isPlaying() {
		return playing;
	}

}
