package example.player;

import java.awt.event.MouseEvent;

import javax.swing.SwingUtilities;


import strokes.Stroke;
import strokes.StrokeShortcuts;
import strokes.criterions.ClickCriterion;
import strokes.criterions.DefaultCriterion;
import strokes.designShortcuts.DesignShortcuts;

/**
 * A standard Swing GUI for a music player
 * augmented with stroke shortcuts. This example
 * is used in the <a href="http://www.lri.fr/~appert/StrokeShortcuts"><i>StrokeShortcuts</i>' tutorial</a>.
 * 
 * @author Caroline Appert
 */
public class StrokePlayer {

	public static void initGUI() {
		
		/****************************************************************/
		/************ Build the standard Swing GUI **********************/
		/****************************************************************/
		
		SimplePlayer playerFrame = new SimplePlayer();
		playerFrame.setLocationRelativeTo(null);
		playerFrame.setVisible(true);
		
		/* ***************************************************************
		 * Designing the mappings (command, stroke) using the 
		 * Design Shortcuts application. The set of shortcuts for this 
		 * music player application is saved in a file "player.strokes".
		 * ***************************************************************/
		/*
		DesignShortcuts designStrokesSet = new DesignShortcuts();
		designStrokesSet.addWindow(playerFrame);
		designStrokesSet.addWindow(playerFrame.dialogAbout);
		designStrokesSet.setVisible(true);
		*/
		
		/****************************************************************/
		/************** Enable stroke shortcuts *************************/
		/****************************************************************/
		
		StrokeShortcuts manager = new StrokeShortcuts();
		manager.addWindow(playerFrame);
		manager.addWindow(playerFrame.dialogAbout);
		
		// Careful, mappings must be registered after having registered the windows
		// because the method addShortcuts browse the graphical hierarchy to enhance
		// menu items and tooltips.
		manager.addShortcuts("player.strokes", true, true);
		
		manager.enableStrokesSheet();
		manager.enableStrokes(playerFrame, true, true);
		manager.enableStrokes(playerFrame.dialogAbout, true, true);
		manager.disableStrokes(playerFrame.sliderSong);
		manager.disableStrokes(playerFrame.sliderVolume);
		manager.setCriterion(playerFrame.playlist, new DefaultCriterion() {
			public boolean startStroke(MouseEvent event) {
				return event.getButton() == MouseEvent.BUTTON3;
			}
		});
		
		
		/* ***************************************************************
		 * Stroke shortcuts can also be registered, shortcut by shortcut, 
		 * without relying on the Design Shortcuts GUI
		 * ***************************************************************/
		/*
		Stroke zigzag = Stroke.zigzag();
		Stroke arche = Stroke.arche();
		Stroke line = Stroke.line();
		Stroke corner = Stroke.corner();
		Stroke mirrorArche = arche.mirrorStroke(Stroke.HORIZONTAL_MIRROR);
		
		manager.addShortcut("Ok", zigzag, true, true);
		manager.addShortcut("Cancel", zigzag.mirrorStroke(Stroke.HORIZONTAL_MIRROR), true, true);
		
		manager.addShortcut("About", Stroke.spiral(1.0), true, true);
		manager.addShortcut("Preferences", arche.append(mirrorArche), true, true);
		manager.addShortcut("Quit", Stroke.helix(2, 40, 30), true, true);
		
		manager.addShortcut("Open playlist", Stroke.arc(Math.PI/4, 3 * Math.PI / 2, false), true, true);
		manager.addShortcut("Save playlist", corner, true, true);
		manager.addShortcut("Import Folder", zigzag.append(zigzag), true, true);
		manager.addShortcut("Remove selected songs", Stroke.loop(), true, true);
		
		manager.addShortcut("Play", zigzag.mirrorStroke(Stroke.VERTICAL_MIRROR).rotateStroke(- Math.PI / 2), true, true);
		manager.addShortcut("Pause", zigzag.rotateStroke(Math.PI / 2), true, true);
		manager.addShortcut("Stop", zigzag.append(Stroke.line().rotateStroke(Math.PI/3)), true, true);
		manager.addShortcut("Fast Forward", Stroke.arche(), true, true);
		manager.addShortcut("Rewind", arche.mirrorStroke(Stroke.VERTICAL_MIRROR), true, true);
		
		manager.addShortcut("Volume Up", line.rotateStroke(3 * Math.PI/2), true, true);
		manager.addShortcut("Volume Down", line.rotateStroke(Math.PI/2), true, true);
		manager.addShortcut("Next", line, true, true);
		manager.addShortcut("Previous", line.rotateStroke(Math.PI) , true, true);
		*/
	}
	
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				initGUI();
			}
		});
		
	}
}
